from __future__ import annotations as _annotations

import asyncio

from acprouter import AppSettings, TelegramGateway


async def main() -> None:
    settings = AppSettings.from_env()
    gateway = TelegramGateway.from_settings(settings)
    await gateway.run()


if __name__ == "__main__":
    asyncio.run(main())



from __future__ import annotations as _annotations

import asyncio
from dataclasses import dataclass, field
from pathlib import Path
from typing import Literal

from acp.schema import (
    PermissionOption,
    RequestPermissionResponse,
    SessionConfigOptionBoolean,
    SessionConfigOptionSelect,
)

__all__ = (
    "ChatBinding",
    "ChatState",
    "PendingApproval",
    "ResolvedApproval",
    "SessionWorkspace",
    "SelectionCommand",
    "SelectionOption",
)

SelectionOption = SessionConfigOptionBoolean | SessionConfigOptionSelect


@dataclass(slots=True, kw_only=True)
class ChatBinding:
    active_session_id: str | None = None
    aliases: dict[str, str] = field(default_factory=dict)
    available_mode_ids: list[str] = field(default_factory=list)
    current_mode_id: str | None = None
    streaming_enabled: bool | None = None


@dataclass(slots=True, kw_only=True)
class ChatState:
    status_message_id: int | None = None
    status_text: str = ""
    response_message_id: int | None = None
    response_text: str = ""
    plan_message_id: int | None = None
    plan_task_ids: dict[str, int] = field(default_factory=dict)
    plan_task_order: list[str] = field(default_factory=list)
    plan_task_texts: dict[str, str] = field(default_factory=dict)
    next_plan_task_id: int = 1
    tool_message_ids: dict[str, int] = field(default_factory=dict)
    tool_texts: dict[str, str] = field(default_factory=dict)
    agent_text: str = ""
    prompt_in_flight: bool = False
    current_approval_id: str | None = None
    resolved_approval: ResolvedApproval | None = None
    current_model_id: str | None = None
    available_model_ids: list[str] = field(default_factory=list)
    selection_options: list[SelectionOption] = field(default_factory=list)
    dynamic_commands: dict[str, SelectionCommand] = field(default_factory=dict)
    last_response_stream_edit_monotonic: float | None = None
    lock: asyncio.Lock = field(default_factory=asyncio.Lock)


@dataclass(slots=True, kw_only=True)
class PendingApproval:
    approval_id: str
    session_id: str
    tool_call_id: str
    tool_title: str
    preview_text: str
    options: list[PermissionOption]
    future: asyncio.Future[RequestPermissionResponse]


@dataclass(slots=True, frozen=True, kw_only=True)
class ResolvedApproval:
    tool_call_id: str
    tool_title: str
    label: str
    message_id: int


@dataclass(slots=True, frozen=True, kw_only=True)
class SessionWorkspace:
    cwd: Path


@dataclass(slots=True, frozen=True, kw_only=True)
class SelectionCommand:
    acp_name: str
    telegram_name: str
    description: str
    hint: str | None = None
    source: Literal["mode", "model", "config", "command"] = "command"



from __future__ import annotations as _annotations

import os
import shlex
from dataclasses import dataclass
from pathlib import Path

__all__ = ("AppSettings",)

_DEFAULT_STDIO_BUFFER_LIMIT_BYTES = 50 * 1024 * 1024
_DEFAULT_STREAMING_EDIT_INTERVAL_SECONDS = 1.0


def _required_command_env() -> str:
    preferred = os.environ.get("ACPROUTER_COMMAND")
    if preferred is not None and preferred.strip() != "":
        return preferred
    legacy = os.environ.get("ACPROUTER_AGENT_COMMAND")
    if legacy is not None and legacy.strip() != "":
        return legacy
    raise RuntimeError(
        "Missing required environment variable: ACPROUTER_COMMAND (legacy: ACPROUTER_AGENT_COMMAND)"
    )


def _required_env(name: str) -> str:
    value = os.environ.get(name)
    if value is None or value.strip() == "":
        raise RuntimeError(f"Missing required environment variable: {name}")
    return value


def _env_bool(name: str, *, default: bool) -> bool:
    raw = os.environ.get(name)
    if raw is None:
        return default
    normalized = raw.strip().lower()
    if normalized in {"1", "true", "on", "yes"}:
        return True
    if normalized in {"0", "false", "off", "no"}:
        return False
    raise RuntimeError(f"{name} must be one of: true, false, on, off, 1, 0, yes, no")


def _validate_surface_env() -> None:
    raw = os.environ.get("ACPROUTER_SURFACE")
    if raw is None or raw.strip() == "" or raw.strip().lower() == "telegram":
        return
    raise RuntimeError("ACPROUTER_SURFACE is no longer supported; acprouter only supports Telegram")


@dataclass(slots=True, frozen=True, kw_only=True)
class AppSettings:
    telegram_api_id: int
    telegram_api_hash: str
    telegram_bot_token: str
    telegram_session_name: str
    telegram_business_connection_id: str | None
    acp_command: tuple[str, ...]
    workspace_root: Path
    state_dir: Path
    acp_cwd: Path
    acp_stdio_buffer_limit_bytes: int
    enable_host_tools: bool
    streaming_default: bool
    streaming_edit_interval_seconds: float

    @classmethod
    def from_env(cls) -> AppSettings:
        workspace_root = (
            Path(os.environ.get("ACPROUTER_WORKSPACE_ROOT", os.getcwd())).expanduser().resolve()
        )
        state_dir = Path(
            os.environ.get("ACPROUTER_STATE_DIR", workspace_root / ".acprouter-state")
        ).expanduser()
        acp_cwd = Path(os.environ.get("ACPROUTER_AGENT_CWD", workspace_root)).expanduser().resolve()
        _validate_surface_env()
        command = tuple(shlex.split(_required_command_env()))
        if not command:
            raise RuntimeError("ACPROUTER_COMMAND must not be empty")
        return cls(
            telegram_api_id=int(_required_env("TELEGRAM_API_ID")),
            telegram_api_hash=_required_env("TELEGRAM_API_HASH"),
            telegram_bot_token=_required_env("TELEGRAM_BOT_TOKEN"),
            telegram_session_name=os.environ.get("ACPROUTER_TELEGRAM_SESSION", "acprouter-bot"),
            telegram_business_connection_id=(
                os.environ.get("ACPROUTER_TELEGRAM_BUSINESS_CONNECTION_ID", "").strip() or None
            ),
            acp_command=command,
            workspace_root=workspace_root,
            state_dir=state_dir.resolve(),
            acp_cwd=acp_cwd,
            acp_stdio_buffer_limit_bytes=int(
                os.environ.get(
                    "ACPROUTER_STDIO_BUFFER_LIMIT_BYTES",
                    str(_DEFAULT_STDIO_BUFFER_LIMIT_BYTES),
                )
            ),
            enable_host_tools=_env_bool("ACPROUTER_ENABLE_HOST_TOOLS", default=True),
            streaming_default=_env_bool("ACPROUTER_STREAMING_DEFAULT", default=False),
            streaming_edit_interval_seconds=float(
                os.environ.get(
                    "ACPROUTER_STREAMING_EDIT_INTERVAL_SECONDS",
                    str(_DEFAULT_STREAMING_EDIT_INTERVAL_SECONDS),
                )
            ),
        )



from __future__ import annotations as _annotations

import asyncio
import signal
import sys
from pathlib import Path

import pytest
from acp.schema import EnvVariable

from acprouter.terminals import _BUFFER_LIMIT, TerminalManager, _TerminalHandle
from acprouter.workspace import WorkspaceManager


@pytest.mark.asyncio
async def test_workspace_manager_reads_and_writes_relative_to_session_cwd(tmp_path: Path) -> None:
    workspace = WorkspaceManager(root=tmp_path)
    session_cwd = tmp_path / "project"
    workspace.bind_session("session-1", session_cwd)

    await workspace.write_text_file("session-1", "notes/hello.txt", "hello\nworld\n")
    response = await workspace.read_text_file("session-1", "notes/hello.txt")

    assert response.content == "hello\nworld\n"
    assert (session_cwd / "notes" / "hello.txt").read_text(encoding="utf-8") == "hello\nworld\n"


@pytest.mark.asyncio
async def test_workspace_manager_supports_line_windowing(tmp_path: Path) -> None:
    workspace = WorkspaceManager(root=tmp_path)
    workspace.bind_session("session-1", tmp_path)
    path = tmp_path / "README.md"
    path.write_text("one\ntwo\nthree\nfour\n", encoding="utf-8")

    response = await workspace.read_text_file("session-1", "README.md", line=2, limit=2)

    assert response.content == "two\nthree"


@pytest.mark.asyncio
async def test_workspace_manager_result_helpers_preserve_paths_and_previous_content(
    tmp_path: Path,
) -> None:
    workspace = WorkspaceManager(root=tmp_path)
    workspace.bind_session("session-1", tmp_path)
    (tmp_path / "main.py").write_text("print('old')\n", encoding="utf-8")

    write_result = await workspace.write_file_result("session-1", "main.py", "print('new')\n")
    read_result = await workspace.read_file_result("session-1", "main.py")

    assert write_result.path == tmp_path / "main.py"
    assert write_result.old_text == "print('old')\n"
    assert write_result.new_text == "print('new')\n"
    assert read_result.path == tmp_path / "main.py"
    assert read_result.content == "print('new')\n"


def test_workspace_manager_defaults_session_cwd_to_root(tmp_path: Path) -> None:
    workspace = WorkspaceManager(root=tmp_path)

    assert workspace.session_cwd("missing-session") == tmp_path
    assert workspace.resolve_path("missing-session", "README.md") == tmp_path / "README.md"


@pytest.mark.asyncio
async def test_workspace_manager_rejects_workspace_escape(tmp_path: Path) -> None:
    workspace = WorkspaceManager(root=tmp_path)
    workspace.bind_session("session-1", tmp_path)

    with pytest.raises(PermissionError, match="Path escapes workspace root"):
        await workspace.read_text_file("session-1", "../outside.txt")


@pytest.mark.asyncio
async def test_terminal_manager_captures_output_and_exit_status(tmp_path: Path) -> None:
    manager = TerminalManager(default_cwd=tmp_path)
    created = await manager.create_terminal(
        sys.executable,
        args=["-c", "print('hello from terminal')"],
        cwd=str(tmp_path),
        env=[EnvVariable(name="ACPROUTER_TEST_ENV", value="1")],
    )

    waited = await manager.wait_for_terminal_exit(created.terminal_id)
    output = await manager.terminal_output(created.terminal_id)
    snapshot = manager.terminal_snapshot(created.terminal_id)
    first_completion = manager.mark_completion_reported(created.terminal_id)
    second_completion = manager.mark_completion_reported(created.terminal_id)
    await manager.release_terminal(created.terminal_id)

    assert waited.exit_code == 0
    assert "hello from terminal" in output.output
    assert output.exit_status is not None
    assert output.exit_status.exit_code == 0
    assert "hello from terminal" in snapshot.output
    assert first_completion is True
    assert second_completion is False


@pytest.mark.asyncio
async def test_terminal_manager_kills_long_running_process(tmp_path: Path) -> None:
    manager = TerminalManager(default_cwd=tmp_path)
    created = await manager.create_terminal(
        sys.executable,
        args=["-c", "import time; time.sleep(30)"],
        cwd=str(tmp_path),
    )

    await manager.kill_terminal(created.terminal_id)
    waited = await manager.wait_for_terminal_exit(created.terminal_id)
    await manager.release_terminal(created.terminal_id)

    assert waited.exit_code is not None or waited.signal is not None


@pytest.mark.asyncio
async def test_terminal_manager_reports_signal_status_and_truncates_output(tmp_path: Path) -> None:
    manager = TerminalManager(default_cwd=tmp_path)

    class _FakeProcess:
        returncode = -signal.SIGTERM

    handle = _TerminalHandle(
        process=_FakeProcess(),
        command_line="fake",
        cwd=tmp_path,
        output=bytearray(b"hello"),
    )
    manager._handles["term"] = handle

    output = await manager.terminal_output("term")
    snapshot = manager.terminal_snapshot("term")

    assert output.exit_status is not None
    assert output.exit_status.signal == "SIGTERM"
    assert snapshot.signal == "SIGTERM"

    class _FakeStream:
        def __init__(self) -> None:
            self.chunks = [b"x" * (_BUFFER_LIMIT + 10), b""]

        async def read(self, size: int) -> bytes:
            del size
            await asyncio.sleep(0)
            return self.chunks.pop(0)

    await manager._capture_output(handle, _FakeStream())

    assert handle.truncated is True
    assert len(handle.output) == _BUFFER_LIMIT



from __future__ import annotations as _annotations

print("Examples:")
print("- examples/telegram_gateway.py")
print("- examples/pydantic_acp_agent.py")
print("- examples/langchain_acp_graph.py")
print("- examples/acprouter_with_acpkit_instance.py")
print("- examples/acpremote_bridge_server.py")
print("- examples/acprouter_remote_acp.py")
